require_relative 'player'
require_relative 'die'
require_relative 'game_turn'
require_relative 'treasure_trove'
module StudioGame
  class Game
    attr_reader :title

    def initialize(title)
      @title = title
      @players = []
    end
    
    def add_player(a_p)
      @players << a_p
    end

    def load_players(file)
      File.readlines(file).each do |line|
        add_player(Player.from_csv(line))
      end
    end

    def save_high_scores(to_file="high_scores.txt")
      File.open(to_file, "w") do |file|
        file.puts "#{title} High Scores:"
          @players.sort.each do |player|
          file.puts player.high_scores
        end
      end
    end
    
    def play(rounds)

        puts "There are #{@players.size} players in #{@title}."
        @players.each {|play| puts play}

        treasures = TreasureTrove::TREASURES
        puts "\nThere are #{treasures.size} treasures to be found:"
        treasures.each { |treasure| puts "A #{treasure.name} is worth #{treasure.points} points"}
        
        1.upto(rounds) do |round|
          if block_given?
            break if yield 
          end
          puts "\nRound #{round}:"
          @players.each do |player|
            GameTurn.take_turn(player)
            puts player 
          end
        end
      
    end

    def print_stats

      strong, wimpy = @players.partition { |player| player.strong? }
      puts "\n#{title} Statistics:\n\n#{strong.count} strong players:"
      strong.each { |player| player.stats}

      puts "\n#{wimpy.count} wimpy players:"
      wimpy.each { |player| player.stats}

      
      puts "\n#{title} High Scores:"
      @players.sort.each do |player|
        puts player.high_scores
      end

      @players.each do |player|
        puts "\n#{player.name}'s point totals:"
        player.each_found_treasure { |treasure| puts "#{treasure.points} total #{treasure.name} points" }

        puts "#{player.points} grand total points"
      end

      puts "\n#{total_points} total points from treasure found"
    end

    def total_points
      @players.reduce(0) { |sum, player| sum + player.points } 
    end
  end
end